import {Injectable, OnDestroy} from '@angular/core'
import {Observable, Subject} from 'rxjs'
import {take, takeUntil} from 'rxjs/operators'
import {AppState} from '../../shared/app-state'
import {HttpClient, HttpResponse} from '@angular/common/http'
import {environment} from '../../../../environments/environment'
import {ApiService} from '../../shared/service/api.service'
import {Permission} from '../../shared/model/user'
import {ErrorService} from '../../shared/service/error.service'
import {AlertService} from '../../shared/service/alert.service'
import {JwtService} from "./jwt.service";

@Injectable({
  providedIn: 'root'
})
export class AccountService implements OnDestroy {
  private destroy$ = new Subject<boolean>()

  constructor(
    private http: HttpClient,
    private api: ApiService,
    private appState: AppState,
    private jwtService: JwtService,
    private errorService: ErrorService,
    private alertService: AlertService
  ) {
  }

  ngOnDestroy(): void {
    this.destroy$.next(true)
    this.destroy$.complete()
  }

  isLoggedIn(): boolean {
    return this.appState.isAuthenticated
  }

  checkAuthenticationStatus() {
    if (!this.appState.authenticatedUser) {
      // Try to get current default group user
      // this.http.get<User>(`${environment.apiUrl}/user/current`, {withCredentials: true})
      //   .pipe(
      //     take(1),
      //     takeUntil(this.destroy$),
      //   ).subscribe(
      //   {
      //     next: user => this.appState.authenticatedUser = user,
      //     error: err => {
      //       if (err.status === 404 || err.status === 403) {
      //         console.warn('No default user is defined on this computer')
      //       } else {
      //         this.errorService.handleError(err)
      //       }
      //     }
      //   })
    }
  }

  login(userId: number, password: string): Observable<any> {
    const subject = new Subject<void>()

    this.http.post<any>(`${environment.apiUrl}/login`, {id: userId, password}, {
      withCredentials: true,
      observe: 'response' as 'body'
    }).pipe(
      take(1),
      takeUntil(this.destroy$)
    ).subscribe({
      next: (response: HttpResponse<void>) => {
        this.loginUser(response)

        subject.next()
        subject.complete()
      },
      error: error => {
        if (error.status === 403) {
          this.alertService.pushError('Les identifiants entrés sont invalides')
        } else {
          this.errorService.handleError(error)
        }

        subject.next()
        subject.complete()
      }
    })

    return subject
  }

  private loginUser(response: HttpResponse<void>) {
    const authorization = response.headers.get("Authorization")
    const jwt = this.jwtService.parseJwt(authorization)

    this.appState.authenticateUser(jwt)
  }

  logout(success: () => void) {
    this.api.get<void>('/logout', true).pipe(
      take(1),
      takeUntil(this.destroy$)
    )
      .subscribe({
        next: () => {
          this.appState.resetAuthenticatedUser()
          this.checkAuthenticationStatus()
          success()
        },
        error: err => this.errorService.handleError(err)
      })
  }

  hasPermission(permission: Permission): boolean {
    return this.appState.authenticatedUser && this.appState.authenticatedUser.permissions.indexOf(permission) >= 0
  }

  private setLoggedInUserFromApi() {
    // this.api.get<User>('/user/current', true)
    //   .pipe(
    //     take(1),
    //     takeUntil(this.destroy$)
    //   )
    //   .subscribe({
    //     next: user => {
    //       this.appState.authenticatedUser = user
    //       At this point the loading wheel should be visible
    // globalLoadingWheel.hide()
    // },
    // error: err => this.errorService.handleError(err)
    // })
    console.warn("REMOVE THIS")
  }
}
